# from .condition_occurrence import Condition_Occurrence
# from .device_exposure import Device_Exposure
# from .drug_exposure import Drug_Exposure
# from .fact_relationship import Fact_Relationship
# from .measurement import Measurement
# from .note_nlp import Note_NLP
# from .note import Note
# from .observation_period import Observation_Period
# from .observation import Observation
# from .person import Person
# from .procedure_occurrence import Procedure_Occurrence
# from .specimen import Specimen
# from .survey_conduct import Survey_Conduct
# from .visit_occurrence import Visit_Occurrence
# from .visit_detail import Visit_Detail
# from .modifiable_table import Modifiable_Table
# from .death import Death

# for concept_linked_table in [Observation, Measurement]:
#     concept_linked_table.add_concepts()

# __all__ = [Condition_Occurrence, Device_Exposure, Drug_Exposure, Fact_Relationship, Measurement,
#            Note_NLP, Note, Observation_Period, Observation, Person, Procedure_Occurrence, Specimen, 
#            Survey_Conduct, Visit_Occurrence, Visit_Detail, Modifiable_Table, Death]