# from .episode import Episode
# from .episode_event import Episode_Event

# for concept_linked_table in [Episode]:
#     concept_linked_table.add_concepts()
# # from .joined_mappers import Condition_Episode, Systemic_Therapy_Episode, Historical_Surgical_Procedure, Dated_Surgical_Procedure,\
# #                             Radiation_Therapy_Episode, Person_Episodes, Dx_Treat_Start, Dx_RT_Start, Dx_SACT_Start


# __all__ = [Episode, Episode_Event]


# # , Condition_Episode, Systemic_Therapy_Episode, Radiation_Therapy_Episode, Historical_Surgical_Procedure, 
# #            Dated_Surgical_Procedure, Person_Episodes, Dx_Treat_Start, Dx_RT_Start, Dx_SACT_Start]